function [v,P,P_diag,R,quat,epsilon]=KFDYNMEASUPDATE(v,P,P_diag,R,quat,Q_p,Ts,t,w_MC,M)

epsilon=zeros(6,M);

for i=1:M
    H=[eye(3) zeros(3)];
    L=H*P(:,:,i)*H'+Q_p;
    K=P(:,:,i)*H'*inv(L);
    epsilon(:,i)=K*w_MC(:,i);
    v(:,t,i)=v(:,t,i)+epsilon(1:3,i);
    quat(:,t,i)=meas_update_quaternion(quat(:,t,i),epsilon(4:6,i));
    P(:,:,i)=P(:,:,i)-K*H*P(:,:,i);
    P(:,:,i)=(P(:,:,i)+P(:,:,i)')/2;
    P_diag(:,t,i)=diag(P(:,:,i));
end

end

%The content of this function is from Isaac Skogs framework OpenShoe
function [q_out]=meas_update_quaternion(q_in,eta)

% Convert quaternion to a rotation matrix
R=quat2Rot(q_in);

% Correct the rotation matrics
R=(eye(3)-Sw(eta))*R;

% Calculte the corrected quaternions
q_out=Rot2quat(R);

end

%The content of this function is from Isaac Skogs framework OpenShoe
function q=Rot2quat(R)

T = 1 + R(1,1) + R(2,2) + R(3,3);

if T > 10^-8
    
    S = 0.5 / sqrt(T);
    qw = 0.25 / S;
    qx = ( R(3,2) - R(2,3) ) * S;
    qy = ( R(1,3) - R(3,1) ) * S;
    qz = ( R(2,1) - R(1,2) ) * S;
    
else
    
    if (R(1,1) > R(2,2)) && (R(1,1) > R(3,3))
        
        S = sqrt( 1 + R(1,1) - R(2,2) - R(3,3)) * 2; % S=4*qx
        qw = (R(3,2) - R(2,3)) / S;
        qx = 0.25 * S;
        qy = (R(1,2) + R(2,1)) / S;
        qz = (R(1,3) + R(3,1)) / S;
        
    elseif (R(2,2) > R(3,3))
        
        S = sqrt( 1 + R(2,2) - R(1,1) - R(3,3) ) * 2; %S=4*qy
        qw = (R(1,3) - R(3,1)) / S;
        qx = (R(1,2) + R(2,1)) / S;
        qy = 0.25 * S;
        qz = (R(2,3) + R(3,2)) / S;
        
    else
        
        S = sqrt( 1 + R(3,3) - R(1,1) - R(2,2) ) * 2; % S=4*qz
        qw = (R(2,1) - R(1,2)) / S;
        qx = (R(1,3) + R(3,1)) / S;
        qy = (R(2,3) + R(3,2)) / S;
        qz = 0.25 * S;
        
    end
    
end

%Store in vector
q = [qx qy qz qw]';
end

%The content of this function is from Isaac Skogs framework OpenShoe
function R=quat2Rot(q)

p=zeros(6,1);

p(1:4)=q.^2;

p(5)=p(2)+p(3);

if p(1)+p(4)+p(5)~=0
    p(6)=2/(p(1)+p(4)+p(5));
else
    p(6)=0;
end


R(1,1)=1-p(6)*p(5);
R(2,2)=1-p(6)*(p(1)+p(3));
R(3,3)=1-p(6)*(p(1)+p(2));

p(1)=p(6)*q(1);
p(2)=p(6)*q(2);
p(5)=p(6)*q(3)*q(4);
p(6)=p(1)*q(2);

R(1,2)=p(6)-p(5);
R(2,1)=p(6)+p(5);

p(5)=p(2)*q(4);
p(6)=p(1)*q(3);

R(1,3)=p(6)+p(5);
R(3,1)=p(6)-p(5);

p(5)=p(1)*q(4);
p(6)=p(2)*q(3);

R(2,3)=p(6)-p(5);
R(3,2)=p(6)+p(5);

end

%The content of this function is from Isaac Skogs framework OpenShoe
function S=Sw(w)
S=[0 -w(3) w(2); w(3) 0 -w(1); -w(2) w(1) 0];
end