function PlotZUPTEKFVelOrientOnly(p, v, P_diag, quat,zupt,u,N,Ts,i,M)

t=Ts:Ts:N*Ts;

%Plot the IMU Data
figure(1);
subplot(2,1,1)
plot(t,u(1:3,:)')
xlabel('t/[s]','Interpreter','Latex')
ylabel('$a/[m/s^2]$','Interpreter','Latex')
title('Accelerometer measurements','Interpreter','Latex')
legend({'x-axis','y-axis','z-axis'},'Interpreter','Latex')
box on
grid on

subplot(2,1,2)
plot(t,u(4:6,:)'*180/pi)
xlabel('t/[s]','Interpreter','Latex')
ylabel('$y_{\omega}/[^{\circ}/s]$','Interpreter','Latex')
title('Angular rate measurements','Interpreter','Latex')
legend({'x-axis','y-axis','z-axis'},'Interpreter','Latex')
box on
grid on


%Plot the velocity, and where the Zero Velocity Update was applied
figure(2); clf;
subplot(2,1,1)
plot(t,v(1:3,:,i))
title('Velocity','Interpreter','Latex')
xlabel('t/[s]','Interpreter','Latex')
ylabel('|v| [m/s]')
grid on
box on

subplot(2,1,2)
stem(t,zupt)
title('ZUPT active','Interpreter','Latex')
xlabel('t/[s]','Interpreter','Latex')
ylabel('true/false')
grid on
box on


%Plot position trajectory estimate from EKF
figure(4); clf;
for j=1:M
    x_og=p(1,:,j);
    y_og=p(2,:,j);
    z_og=p(3,:,j);
    plot3(x_og,y_og,z_og,'k');
    hold on;
end

scatter3(x_og(1), y_og(1), z_og(1), 'ro');
axis equal;
legend({'Estimated trajectory','Starting point'},'Interpreter','Latex');
xlabel('x/[m]','Interpreter','Latex');
ylabel('y/[m]','Interpreter','Latex');
zlabel('z/[m]','Interpreter','Latex');
hold off;

%Plot the position and velocity covariances
figure(5); clf;
P1=zeros(1,N); P1(:,:)=P_diag(1,:,i);
P2=zeros(1,N); P2(:,:)=P_diag(2,:,i);
P3=zeros(1,N); P3(:,:)=P_diag(3,:,i);
P4=zeros(1,N); P4(:,:)=P_diag(4,:,i);
P5=zeros(1,N); P5(:,:)=P_diag(5,:,i);
P6=zeros(1,N); P6(:,:)=P_diag(6,:,i);

subplot(2,1,1)
plot(t,sqrt([P1;P2;P3]));
title('Velocity standard deviations','Interpreter','Latex')
ylabel('$\sigma$/[m/s]','Interpreter','Latex')
xlabel('t/[s]','Interpreter','Latex')
legend({'$\textbf{v}_x$', '$\textbf{v}_y$','$\textbf{v}_z$'},'Interpreter','Latex')
grid on
box on

subplot(2,1,2)
plot(t,sqrt([P4;P5;P6])'*180/pi)
title('Orientation standard deviations','Interpreter','Latex')
ylabel('$\sigma$/[deg]','Interpreter','Latex')
xlabel('t/[s]','Interpreter','Latex')
legend({'$\eta_1$', '$\eta_2$','$\eta_3$'},'Interpreter','Latex')
grid on
box on

%Plot orientations
eulXYZ=zeros(3,N);

for i=1:N
    eulXYZ(:,i) = quat2eul(quat(:,i)','XYZ');
end

figure(6); clf;
plot(t,eulXYZ(1:3,:))
title('Orientation','Interpreter','Latex')
xlabel('t/[s]','Interpreter','Latex')
ylabel('[radians/s]')
legend({'roll','pitch','yaw'},'Interpreter','Latex');
grid on
box on

end
