clear;
%Read the ground truth position estimate
gt=csvread('fake_ground_truth_CheckpointsXsens2.csv');

%Read the calibrated gyroscope, accelerometer and magnetometer measurements
u=csvread('CheckpointsXsens2.csv');

%Downsample and limit u, gt
N=1900;
u=u(:,1:10:N);
gt=gt(:,1:10:N); Ts=1/10; N=size(gt,2);

%Calculate the magnetic field norm measurements from the
%magnetic field measurements
y_m=zeros(size(gt,2),1);
for i=1:N
    y_m(i)=norm(u(7:9,i));
end

%Center the magnetic field norm measurements around zero
mean_mag_norm=mean(y_m);
y_m=y_m-mean_mag_norm;

%Set room dimensions
margin=2; %distance from trajectory to wall [m]
uL1=max(gt(1,:))+margin; %upper limit x, a.k.a. the position of the right wall [m]
lL1=min(gt(1,:))-margin; %lower limit x, a.k.a. the position of the left wall [m]
uL2=max(gt(2,:))+margin; %uppper limit y, a.k.a. the position of the upper wall [m]
lL2=min(gt(2,:))-margin; %lower limit y, a.k.a. the position of the lower wall [m]
uL3=max(gt(3,:))+margin; %upper limit z, a.k.a. the position of the ceiling [m]
lL3=min(gt(3,:))-margin; %lower limit z, a.k.a. the position of the floor [m]

%Plot the magnetic field measurements in their respective locations
figure(4); clf;
scatter3(gt(1,:),gt(2,:),gt(3,:),50,y_m+mean_mag_norm,'filled');
hold on;
scatter3(gt(1,:),gt(2,:),gt(3,:),50,'ko');
hc=colorbar;
title(hc,'$y_{m}$','Interpreter','Latex');
caxis([0.5 1.4]);
axis equal;
xlim([lL1+1 uL1-1]);
ylim([lL2+1 uL2-1]);
zlim([lL3+1 uL3-1]);
xlabel('$x$/[m]','Interpreter','Latex');
ylabel('$y$/[m]','Interpreter','Latex');
zlabel('$z$/[m]','Interpreter','Latex');

%Plot the trajectory and the sampling locations
figure(1); clf;
plot3(gt(1,:),gt(2,:),gt(3,:),'k');
view(2);
hold on;
xlim([lL1 uL1]);
ylim([lL2 uL2]);
zlim([lL3 uL3]);
scatter3(gt(1,:),gt(2,:),gt(3,:),'ro');
legend({'Trajectory','Sampling Locations'},'Interpreter','Latex');
title(['sampling frequency $f=',num2str(1/Ts),'Hz$'],'Interpreter','Latex');
xlabel('$x/[m]$','Interpreter','Latex');
ylabel('$y/[m]$','Interpreter','Latex');
zlabel('$z/[m]$','Interpreter','Latex');

%Define hyperparameters for the magnetic field map GP prior
sigma_SE=1;
l_SE=0.3;
sigma_y=0.1;

%Number of basis functions used in Reduced-Rank approximation
N_m=2000;

%Define resolution of plots (the higher resolution the smoother, and the
%longer calculation time required. A resolution of 0.1 takes approx 30 min)
res=0.1;

%Calculate Lambda and the order of indices used in the 
%analytic basis functions of the Reduced-Rank Approximation
[Indices, Lambda]=Lambda3D(N_m,uL1,lL1,uL2,lL2,uL3,lL3,sigma_SE,l_SE);

%Find magnetic field map
[m,P_m]=LinearModelGP(gt,y_m,sigma_y,N_m,uL1,lL1,uL2,lL2,uL3,lL3,Indices,Lambda);

% Plot the magnetic field map
figure(2); clf;
plot_projection_mag_field_map3D(m,P_m,Indices,uL1,lL1,uL2,lL2,uL3,lL3,0,0.1);
scatter3(gt(1,:),gt(2,:),y_m,'r');
xlim([lL1 uL1]);
ylim([lL2 uL2]);

Plot3DMap;





