clear;

%Set room dimensions
margin=1.3;
uL1=2.5;
lL1=-2.5;
uL2=2.5;
lL2=-2.5;
uL3=1;
lL3=-1;

%Simulate the ground truth position estimate
Ts=1/5; z=0;
gt=rectangular_walk_ground_truth(Ts,lL1,uL1,lL2,uL2,margin,z);
y=-1.5:0.3:0; x=y; y=[y 0*y]; x=[0*x x]; z=0*x; gt=[x; y; z];
N=size(gt,2);

%Define hyperparameters for the magnetic field map GP prior
sigma_SE=200;
l_SE=1.3;
sigma_y=10;

%Simulate magnetic field norm measurements in the trajectory
mag_field_norm=mvnrnd(zeros(size(gt,2),1),Kern(gt,gt,sigma_SE,l_SE));
y_sim=mag_field_norm+mvnrnd(zeros(size(gt,2),1),sigma_y^2)';

%Make a map based on the ground truth position
N_m=256;
[Indices, Lambda]=Lambda3D(N_m,uL1,lL1,uL2,lL2,uL3,lL3,sigma_SE,l_SE);


%Find magnetic field map
[m,P_m]=LinearModelGP(gt,y_sim,sigma_y,N_m,uL1,lL1,uL2,lL2,uL3,lL3,Indices,Lambda);

% Plot the magnetic field map
figure(1); clf;
plot_projection_mag_field_map3D(m,P_m,Indices,uL1,lL1,uL2,lL2,uL3,lL3,0,0.1)
scatter(gt(1,:),gt(2,:),'r');
xlim([lL1 uL1]);
ylim([lL2 uL2]);







