clear;

%Define hyperparamters GP and RR-GP
lL=-1; %lower bound
uL=1; %upper bound
res=0.001;
x=lL:res:uL;
sigma_SE=1;
l_SE=0.1;
sigma_y=0.2;
b=5; l=5; %Let the number of eigenfunctions be a product of these two
N_m=b*l; %Number of eigenfunctions

%Define hyperparameters experiment
samples=40; %number of samples used to fit the GP and the RR-GP
p=0.2; %proportion used as a margin to avoid samples close to the domain edges

y=mvnrnd(0*x,Kern(x,x,sigma_SE,l_SE)); %The full underlying GP function
y_min=min(y);

%Set fontsize for plots
fontsize=14;


hold on;
samples=40;
sz=size(x,2);

indices=floor(sz*p)+randi(floor(sz*(1-2*p)),samples,1);
x_known=x(indices);
y_known=y(indices)+mvnrnd(zeros(samples,1),sigma_y.^2*eye(samples));

x_star=x;

figure(1);
clf;
%Plot the basis functions, scaled acording to S_SE functionality
PHI=zeros(N_m,size(x,1));
for i=1:size(x,2)
    phi=Phi(x(i),N_m,uL,lL);
    PHI(:,i)=phi;
end

figure(1); clf;

for i=1:b
    for j=1:l
        func_nr=j+(i-1)*l;
        subplot(b,l,func_nr);
        plot(x,PHI(func_nr,:));
        hold on;

    end
end
hold off;

%Standard GP
[y_star_GP, y_star_var_GP]=GaussianProcess(x_known,y_known,x_star,sigma_y,sigma_SE,l_SE);


%Plot GP results
figure(3); clf;
y_star_min_GP=min(y_star_GP);
variances_GP=sqrt(diag(y_star_var_GP));
plot_min=min(y_min,y_star_min_GP)-max(sqrt(diag(y_star_var_GP)));
plot_max=max(max(y_star_GP),max(y))+max(sqrt(diag(y_star_var_GP)));
H=area(x_star(2:end),y_star_GP(2:end)+variances_GP(2:end),plot_min);
hold on;
A=area(x_star(2:end),y_star_GP(2:end)-variances_GP(2:end),plot_min);
set(H,'FaceColor',[0.9 0.9 0.9]);
H.EdgeColor='none';
A.EdgeColor='none';
hold on;
set(A,'FaceColor',[1 1 1]);
plot(x,y,'b');
plot(x_star,y_star_GP,'k');
scatter(x_known,y_known,'r');
legend({'Plus/minus one standard deviation','','True function','Gaussian process approximation','Known samples'},'Interpreter','Latex','Fontsize',fontsize);
hold off;
axis off;
axis on;
ylim([plot_min plot_max]);
xlabel('$x$','Interpreter','Latex','Fontsize',fontsize);
ylabel('$y$','Interpreter','Latex','Fontsize',fontsize);
title('Gaussian process regression','Interpreter','Latex','Fontsize',fontsize);

%Reduced-rank GP
[y_star, y_star_var]=ReducedRankGaussianProcess(x_known,y_known,x_star,uL,lL,sigma_y,N_m,sigma_SE,l_SE);


%Plot red-rank GP results
figure(2); clf;
y_star_min=min(y_star);
variances=sqrt(diag(y_star_var));
plot_min=min(y_min,y_star_min)-max(sqrt(diag(y_star_var)));
plot_max=max(max(y_star),max(y))+max(sqrt(diag(y_star_var)));
H=area(x_star(2:end),y_star(2:end)+variances(2:end),plot_min);
hold on;
A=area(x_star(2:end),y_star(2:end)-variances(2:end),plot_min);
set(H,'FaceColor',[0.729 0.914 0.949]);
H.EdgeColor='none';
A.EdgeColor='none';
hold on;
set(A,'FaceColor',[1 1 1]);
p1=plot(x_star,y_star,'Linewidth',3,'Color',[0.157 0.545 0.827]);
p0=plot(x_star,y_star_GP,'k');
p2=plot(x_star(2:end),y_star_GP(2:end)+variances_GP(2:end),'--k');
plot(x_star(2:end),y_star_GP(2:end)-variances_GP(2:end),'--k');
p3=scatter(x_known,y_known,'r');
legend([p0 H p1 p2 p3],{'$E[y*]$ GP','$\pm \sigma$ RRGP','$E[y*]$ RRGP','$\pm \sigma$ GP','Samples'},'Interpreter','Latex','Fontsize',fontsize);
hold off;
axis off;
axis on;
ylim([plot_min plot_max]);
xlabel('$x$','Interpreter','Latex','Fontsize',fontsize);
ylabel('$y$','Interpreter','Latex','Fontsize',fontsize);
title(['Reduced rank Gaussian process regression with ',num2str(N_m),' basis functions'],'Interpreter','Latex','Fontsize',fontsize);
