clear;

%Define hyperparameters GP
lL=-1; %lower bound
uL=1; %upper bound
res=0.01;
x=lL:res:uL;
sz=size(x,2);
sigma_SE=1;
l_SE=0.1;
sigma_y=0.2;

%Define hyperparameters experiments
N_m_max=100;
N_ms=1:6:36;
tests=size(N_ms,2);
experiments=10;
pred_RedRank_RMSEs=zeros(experiments,tests);
pred_GP_RMSEs=zeros(experiments,1);
samples_meas=100;
samples_test=100;
p=0.1; %proportion used as a margin to avoid samples close to the domain edges
samples=samples_meas+samples_test;

figure(1); clf;

for experiment=1:experiments
    
    %For this experiment, generate random known and test measurements
    %From a simulated gaussian process using the known hyperparameters
    indices=floor(sz*p)+randi(floor(sz*(1-2*p)),samples,1);
    x_all=x(indices);
    f=mvnrnd(zeros(samples,1),Kern(x_all,x_all,sigma_SE,l_SE));
    x_meas=x_all(1:samples_meas);
    x_test=x_all((samples_meas+1):(samples_meas+samples_test));
    f_meas=f(1:samples_meas);
    f_test=f((samples_meas+1):(samples_meas+samples_test));
    y_meas=f_meas+mvnrnd(zeros(samples_meas,1),sigma_y.^2*eye(samples_meas));
    
    %Make a GP prediction in this point for comparison
    [f_test_GP_pred, f_test_GP_var]=GaussianProcess(x_meas,y_meas,x_test,sigma_y,sigma_SE,l_SE);
    
    %Make a full-line GP prediction for visualisation purposes
    [f_GP_pred, f_GP_var]=GaussianProcess(x_meas,y_meas,x,sigma_y,sigma_SE,l_SE);
    
    pred_E=f_test-f_test_GP_pred'; % Prediction Errors
    pred_SE=pred_E.^2;  % Prediction Squared Errors
    pred_MSE=mean(pred_SE);  % Prediction Mean Squared Error
    pred_RMSE = sqrt(pred_MSE);  % Prediction Root Mean Squared Error
    pred_GP_RMSEs(experiment)=pred_RMSE;
    
    for test=1:tests
        %Decide the number of basis functions for this test
        N_m=N_ms(test);
        [f_test_RR_pred, f_test_RR_var]=ReducedRankGaussianProcess(x_meas,y_meas,x_test,uL,lL,sigma_y,N_m,sigma_SE,l_SE);
        pred_E=f_test-f_test_RR_pred'; % Prediction Errors
        pred_SE=pred_E.^2;  % Prediction Squared Errors
        pred_MSE=mean(pred_SE);  % Prediction Mean Squared Error
        pred_RMSE = sqrt(pred_MSE);  % Prediction Root Mean Squared Error
        pred_RedRank_RMSEs(experiment,test)=pred_RMSE;

        %Make a full-line prediction for visualisation purposes
        [f_RR_pred, f_RR_var]=ReducedRankGaussianProcess(x_meas,y_meas,x,uL,lL,sigma_y,N_m,sigma_SE,l_SE);
        subplot(experiments,tests,(experiment-1)*tests+test);
        plot_RR_results(x,x_meas,y_meas,x_test,f_test_RR_pred,f_test_GP_pred,f_RR_pred,f_GP_pred,f_RR_var,f_GP_var);
        title(['experiment: ',num2str(experiment),', N_m: ',num2str(N_m)])
        
    end
    
    
end

PlotResultsExperiments;