clear;
p=load('fake_ground_truth_CheckpointsXsens2.csv');
p=p(:,1:3000); %Limit the trajectory length

%These Monte-Carlo simulations take approximately 5 hours to run
%On a standard laptop

N=size(p,2);
margin=2;
uL1=max(p(1,:))+margin;
lL1=min(p(1,:))-margin;
uL2=max(p(2,:))+margin;
lL2=min(p(2,:))-margin;
uL3=max(p(3,:))+margin;
lL3=min(p(3,:))-margin;

figure(1); clf;
sigma_SE=1;
l_SE_s=[0.1 0.3 1];
sigma_y_s=[0.01 0.1 1];
l=size(l_SE_s,2); b=size(sigma_y_s,2);
for l_SE_index=1:l
    for sigma_y_index=1:b
        subplot(l,b,(l_SE_index-1)*b+sigma_y_index);
        l_SE=l_SE_s(l_SE_index);
        sigma_y=sigma_y_s(sigma_y_index);
        
        %Define hyperparameters experiments
        N_m_max=3000;
        N_ms=1:100:N_m_max;
        tests=size(N_ms,2);
        experiments=2;
        pred_RedRank_RMSEs=zeros(experiments,tests);
        pred_GP_RMSEs=zeros(experiments,1);
        samples_meas=1000;
        samples_test=100;
        samples=samples_meas+samples_test;
        
        for experiment=1:experiments
            
            hold on;
            indices=randperm(N,samples);
            x_meas=p(:,indices(1:samples_meas));
            x_test=p(:,indices(samples_meas+1:samples));
            
            
            
            %Draw random samples from the prior
            x_all=[x_meas x_test];
            f=mvnrnd(zeros(samples,1),Kern(x_all,x_all,sigma_SE,l_SE));
            f_meas=f(1:samples_meas);
            f_test=f(samples_meas+1:samples);
            y_meas=f_meas+mvnrnd(zeros(samples_meas,1),sigma_y.^2*eye(samples_meas));
            
            %Standard GP
            [f_test_GP_pred, f_test_GP_var]=GaussianProcess(x_meas,y_meas,x_test,sigma_y,sigma_SE,l_SE);

            
            pred_E=f_test-f_test_GP_pred'; % Prediction Errors
            pred_SE=pred_E.^2;  % Prediction Squared Errors
            pred_MSE=mean(pred_SE);  % Prediction Mean Squared Error
            pred_RMSE = sqrt(pred_MSE);  % Prediction Root Mean Squared Error
            pred_GP_RMSEs(experiment)=pred_RMSE;
            
            for test=1:tests
                N_m=N_ms(test);
                [Indices, Lambda]=Lambda3D(N_m,uL1,lL1,uL2,lL2,uL3,lL3,sigma_SE,l_SE);
                %Reduced-rank GP
                P_m=Lambda;
                m=zeros(N_m,1);
                
                for t=1:samples_meas
                    x_t=x_meas(:,t);
                    phi_t=Phi3D(x_t,N_m,uL1,lL1,uL2,lL2,uL3,lL3,Indices);
                    S_t=phi_t*P_m*phi_t'+sigma_y^2;
                    K_t=P_m*phi_t'*inv(S_t);
                    m=m+K_t*(y_meas(t)-phi_t*m);
                    P_m=P_m-K_t*S_t*K_t';
                    P_m=1/2*(P_m+P_m');
                end
                
                phi_test=Phi3D(x_test,N_m,uL1,lL1,uL2,lL2,uL3,lL3,Indices);
                f_test_RR_pred=phi_test*m;
                f_test_RR_covar=phi_test*P_m*phi_test';
                                
                pred_E=f_test-f_test_RR_pred'; % Prediction Errors
                pred_SE=pred_E.^2;  % Prediction Squared Errors
                pred_MSE=mean(pred_SE);  % Prediction Mean Squared Error
                pred_RMSE = sqrt(pred_MSE);  % Prediction Root Mean Squared Error
                pred_RedRank_RMSEs(experiment,test)=pred_RMSE;
                
            end
            disp(['Experiment ',num2str(experiment),' completed']);
        end
        disp(['Round ',num2str((l_SE_index-1)*b+sigma_y_index),' completed']);
        PlotResultsExperimentsVaryingHyperparams;
    end
end