clear;
%Set room dimensions
margin=2; %distance from trajectory to wall [m]
uL1=4; %upper limit x, a.k.a. the position of the right wall [m]
lL1=-4; %lower limit x, a.k.a. the position of the left wall [m]
uL2=5; %uppper limit y, a.k.a. the position of the upper wall [m]
lL2=-5; %lower limit y, a.k.a. the position of the lower wall [m]
uL3=2; %upper limit z, a.k.a. the position of the ceiling [m]
lL3=-2; %lower limit z, a.k.a. the position of the floor [m]

%Simulate the ground truth position estimate
Ts=1/10; %Sampling time [s]
v=1.4; %Walking speed [m/s]
z=0; %z position of trajectory
gt=rectangular_walk_ground_truth(Ts,lL1,uL1,lL2,uL2,margin,v,z);
N=size(gt,2);

%Plot the trajectory and the sampling locations
figure(1); clf;
plot3(gt(1,:),gt(2,:),gt(3,:),'k');
view(2);
hold on;
xlim([lL1 uL1]);
ylim([lL2 uL2]);
zlim([lL3 uL3]);
scatter3(gt(1,:),gt(2,:),gt(3,:),'ro');
legend({'Trajectory','Sampling Locations'},'Interpreter','Latex');
title(['Walking speed $v=',num2str(v),'ms^{-1}$, sampling frequency $f=',num2str(1/Ts),'Hz$'],'Interpreter','Latex');
xlabel('$x/[m]$','Interpreter','Latex');
ylabel('$y/[m]$','Interpreter','Latex');
zlabel('$z/[m]$','Interpreter','Latex');

%Define hyperparameters for the magnetic field map GP prior
sigma_SE=1;
l_SE=0.6;
sigma_y=0.1;

%Number of basis functions used in Reduced-Rank approximation
N_m=1000;

%Define resolution of plots (the higher resolution the smoother, and the
%longer calculation time required. A resolution of 0.1 takes approx 30 min)
res=0.1;

%Simulate magnetic field norm in the trajectory
mag_field_norm=mvnrnd(zeros(size(gt,2),1),Kern(gt,gt,sigma_SE,l_SE));

%Simulate magnetic field norm measurements in the trajectory
y_sim=mag_field_norm+mvnrnd(0,sigma_y^2,size(gt,2))';

%Calculate Lambda and the order of indices used in the 
%analytic basis functions of the Reduced-Rank Approximation
[Indices, Lambda]=Lambda3D(N_m,uL1,lL1,uL2,lL2,uL3,lL3,sigma_SE,l_SE);

%Find magnetic field map
[m,P_m]=LinearModelGP(gt,y_sim,sigma_y,N_m,uL1,lL1,uL2,lL2,uL3,lL3,Indices,Lambda);

% Plot the magnetic field map
figure(2); clf;
plot_projection_mag_field_map3D(m,P_m,Indices,uL1,lL1,uL2,lL2,uL3,lL3,0,0.1)
scatter3(gt(1,:),gt(2,:),mag_field_norm,'r');
xlim([lL1 uL1]);
ylim([lL2 uL2]);

Plot3DMap;






