
%Make a illustration of the specific magnetic field estimates in each
%position in the box
grid_res=0.5;
layer_res=0.5;
[X1,X2] = meshgrid(lL1:grid_res:uL1,lL2:grid_res:uL2);
length=size(X1,1); width=size(X1,2);
mu=X1;
CoVarMu=X1;
zs=[lL3:layer_res:(uL3), uL3];
layers=size(zs,2);

%Set up movie object
vidObj = VideoWriter('3D_Magnetic_field_visual.avi');
vidObj.Quality = 100;
vidObj.FrameRate = layers./10;
open(vidObj);
set(gca,'nextplot','replacechildren');
set(gca, 'visible', 'off');
F(layers) = struct('cdata',[],'colormap',[]);

for layer=1:layers
    z=zs(layer);
    for i=1:length
        for j=1:width
            p_s=[X1(i,j); X2(i,j); z];
            phi_s=Phi3D(p_s,N_m,uL1,lL1,uL2,lL2,uL3,lL3,Indices);
            mu_temp=phi_s*m(:,end);
            var=phi_s*P_m(:,:,end)*phi_s';
            mu(i,j)=mu_temp;
            CoVarMu(i,j)=var;
        end
        disp(['calculated slice ',num2str(i),' of ',num2str(length)]);
    end
    
    %Plot the result
    figure(1); clf;
    mu_corrected=(mu+mean_mag_norm);
    Opacity=-CoVarMu;
    s1=surf(X1,X2,z*ones(size(X1)),mu_corrected,'AlphaData',Opacity,'FaceAlpha','flat');
    s1.EdgeColor = 'none';
    colormap default;
    hold on;
    plot3(p(1,:),p(2,:),p(3,:));
    hc=colorbar;
    title(hc,'$\mu T$','Interpreter','Latex');
    xlabel('$x$/[m]','Interpreter','Latex');
    ylabel('$y$/[m]','Interpreter','Latex');
    zlabel('$\left \| \textbf{H} \right \|$/[$\mu T$]','Interpreter','Latex');
    title('2D map of magnetic field anomalies Reduced Rank Gaussian Process');
    axis equal;
    xlim([lL1 uL1]);
    ylim([lL2 uL2]);
    zlim([lL3 (uL3+layer_res)]);
    
    %%Store plot to video
    F(layer) = getframe;
    currFrame = getframe(gcf);
    writeVideo(vidObj,currFrame);
    
end


close(vidObj);