function p=rectangular_walk_ground_truth(Ts,lL1,uL1,lL2,uL2,margin,v,z)
%Ts - Sampling time [s]
%lL1 - lower limit x, a.k.a. the position of the left wall [m]
%uL1 - upper limit x, a.k.a. the position of the right wall [m]
%lL2 - lower limit y, a.k.a. the position of the lower wall [m]
%uL2 - uppper limit y, a.k.a. the position of the upper wall [m]
%lL3 - lower limit y, a.k.a. the position of the lower wall [m]
%uL3 - uppper limit y, a.k.a. the position of the upper wall [m]
%margin - distance from trajectory to wall [m]
%v - walking speed [m/s]
%z - position along z-axis of trajectory [m]

%length - Length of trajectory in meter
length=uL1-lL1-2*margin;

%height - %Height of trajectory in meter
height=uL2-lL2-2*margin;

%Distance traversed per sample
delta_p=v*Ts;

%Make the four edges of the rectangle
p1=[delta_p:delta_p:(length); zeros(1,floor(length/delta_p)); z+zeros(1,floor(length/delta_p))];
p2=[zeros(1,floor(height/delta_p)); delta_p:delta_p:(height); z+zeros(1,floor(height/delta_p))]+p1(:,end);
p3=[-(delta_p:delta_p:(length)); zeros(1,floor(length/delta_p)); z+zeros(1,floor(length/delta_p))]+p2(:,end);
p4=[zeros(1,floor(height/delta_p)); -(delta_p:delta_p:(height)); z+zeros(1,floor(height/delta_p))]+p3(:,end);

%Concatenate the edges to make full rectangle
p=[p1 p2 p3 p4];

%Repeat the rectangular walking pattern to make four laps
p=[p p p p];

%Shift all positions according to the initial position
p=p+[lL1+margin; lL2+margin; z];

end