%The content of this function is a modified version of the "ZUPTaidedINS"
%function with in Isaac Skogs "OpenShoE" software, where functionaliy
%that is not necessary for this project has been removed,
%And variables have been renamed to fit the notation in the report.

function [Q,R_v,v,P,P_diag,R,quat,acc_t]=ZUPTEKFInit(u,Ts,N,M)

Q=zeros(6);
alpha=1.2;
sigma_acc =0.1*alpha*[1 1 1]';
sigma_gyro =0.05*alpha*[1 1 1]'*pi/180;
sigma_vel=0.1*alpha*[1 1 1];
Q(1:3,1:3)=diag(sigma_acc.^2)*Ts;
Q(4:6,4:6)=diag(sigma_gyro.^2)*Ts;
R_v=diag(sigma_vel.^2)*Ts;

%Allocate memory for trajectories
P=zeros(6,6,M);
P_diag=zeros(6,N,M);
v=zeros(3,N,M);

for i=1:M
    P(:,:,i)=0.1*alpha*eye(6)*Ts;
    P_diag(:,1,i)=diag(P(:,:,i));
end

quat_init=init_orientation(u);
R=zeros(3,3,N);
quat=zeros(4,N,M);
R(:,:,1)=quat2Rot(quat_init);
for i=1:M
quat(:,1,i)=quat_init;
end
acc_t=zeros(3,N,M);

end

%The content of this function is from Isaac Skogs framework OpenShoe
function [quat]=init_orientation(u)

% Under the assumption that the system is stationary during the first 20
% samples, the initial roll and pitch is calculate from the 20 first
% accelerometer readings.
f_u=mean(u(1,1:20));
f_v=mean(u(2,1:20));
f_w=mean(u(3,1:20));
roll=atan2(-f_v,-f_w);
pitch=atan2(f_u,sqrt(f_v^2+f_w^2))+pi;
% Set the attitude vector
attitude=[roll pitch 0]';

% Calculate quaternion corresponing to the initial attitude
R=Euler2Rot(attitude)';
quat=Rot2quat(R);
end


%The content of this function is from Isaac Skogs framework OpenShoe
function [q_out]=meas_update_quaternion(q_in,eta)

% Convert quaternion to a rotation matrix
R=quat2Rot(q_in);

% Correct the rotation matrics
R=(eye(3)-Sw(eta))*R;

% Calculte the corrected quaternions
q_out=Rot2quat(R);

end

%The content of this function is taken from Isaac Skogs framework OpenShoe
function q_new=dyn_update_quaternion(q_old,eta)

P=eta(1);
Q=eta(2);
R=eta(3);

OMEGA=zeros(4);
OMEGA(1,1:4)=0.5*[0 R -Q P];
OMEGA(2,1:4)=0.5*[-R 0 P Q];
OMEGA(3,1:4)=0.5*[Q -P 0 R];
OMEGA(4,1:4)=0.5*[-P -Q -R 0];

v=norm(eta);
if v~=0
    q_new=(cos(v/2)*eye(4)+2/v*sin(v/2)*OMEGA )*q_old;
    q_new=q_new./norm(q_new); %Normalization because an approximation to the quaternion
    %multiplication is used. If the proper quaternion multiplication
    %was implemented, no normalization would be necessary.
else
    q_new=q_old;
end

end

%The content of this function is from Isaac Skogs framework OpenShoe
function q=Rot2quat(R)

T = 1 + R(1,1) + R(2,2) + R(3,3);

if T > 10^-8
    
    S = 0.5 / sqrt(T);
    qw = 0.25 / S;
    qx = ( R(3,2) - R(2,3) ) * S;
    qy = ( R(1,3) - R(3,1) ) * S;
    qz = ( R(2,1) - R(1,2) ) * S;
    
else
    
    if (R(1,1) > R(2,2)) && (R(1,1) > R(3,3))
        
        S = sqrt( 1 + R(1,1) - R(2,2) - R(3,3)) * 2; % S=4*qx
        qw = (R(3,2) - R(2,3)) / S;
        qx = 0.25 * S;
        qy = (R(1,2) + R(2,1)) / S;
        qz = (R(1,3) + R(3,1)) / S;
        
    elseif (R(2,2) > R(3,3))
        
        S = sqrt( 1 + R(2,2) - R(1,1) - R(3,3) ) * 2; %S=4*qy
        qw = (R(1,3) - R(3,1)) / S;
        qx = (R(1,2) + R(2,1)) / S;
        qy = 0.25 * S;
        qz = (R(2,3) + R(3,2)) / S;
        
    else
        
        S = sqrt( 1 + R(3,3) - R(1,1) - R(2,2) ) * 2; % S=4*qz
        qw = (R(2,1) - R(1,2)) / S;
        qx = (R(1,3) + R(3,1)) / S;
        qy = (R(2,3) + R(3,2)) / S;
        qz = 0.25 * S;
        
    end
    
end

%Store in vector
q = [qx qy qz qw]';
end

%The content of this function is from Isaac Skogs framework OpenShoe
function R=quat2Rot(q)

p=zeros(6,1);

p(1:4)=q.^2;

p(5)=p(2)+p(3);

if p(1)+p(4)+p(5)~=0
    p(6)=2/(p(1)+p(4)+p(5));
else
    p(6)=0;
end


R(1,1)=1-p(6)*p(5);
R(2,2)=1-p(6)*(p(1)+p(3));
R(3,3)=1-p(6)*(p(1)+p(2));

p(1)=p(6)*q(1);
p(2)=p(6)*q(2);
p(5)=p(6)*q(3)*q(4);
p(6)=p(1)*q(2);

R(1,2)=p(6)-p(5);
R(2,1)=p(6)+p(5);

p(5)=p(2)*q(4);
p(6)=p(1)*q(3);

R(1,3)=p(6)+p(5);
R(3,1)=p(6)-p(5);

p(5)=p(1)*q(4);
p(6)=p(2)*q(3);

R(2,3)=p(6)-p(5);
R(3,2)=p(6)+p(5);

end

%The content of this function is from Isaac Skogs framework OpenShoe
function S=Sw(w)
S=[0 -w(3) w(2); w(3) 0 -w(1); -w(2) w(1) 0];
end

%The content of this function is from Isaac Skogs framework OpenShoe
function R=Euler2Rot(ang)
cr=cos(ang(1));
sr=sin(ang(1));

cp=cos(ang(2));
sp=sin(ang(2));

cy=cos(ang(3));
sy=sin(ang(3));

R=[cy*cp sy*cp -sp;
    -sy*cr+cy*sp*sr cy*cr+sy*sp*sr cp*sr;
    sy*sr+cy*sp*cr -cy*sr+sy*sp*cr cp*cr];

end



